(* 05/23/01 *)
<* +M2EXTENSIONS *>
<* +M2ADDTYPES   *>
MODULE PGPTest;

  IMPORT SYSTEM, InOut;
  IMPORT WinDef, WinBase, WinNT;
  IMPORT pgpBase, pgpPubTypes, pgpKeys, pgpUtilities, pgpEncode, pgpOptionList, pgpPFLErrors;

CONST
  PGP_USERID      = "VICTOR_TEST2";
  PGP_SRCFILENAME = "pgp.cmd";
  PGP_PGPFILENAME = "pgp.cmd.pgp";
  PGP_ORGFILENAME = "pgp.cmd.org";

VAR
  pgp_resp            : pgpBase.PGPError;
  pgp_numKeys         : pgpBase.PGPUInt32;                    (* The number of keys in <pgp_foundUserKeys> keyset *)
  pgp_context         : pgpPubTypes.PGPContextRef;            (* Current PGP context                              *)
  pgp_defaultKeyset   : pgpPubTypes.PGPKeySetRef;             (* Default keys set for <pgp_context>               *)
  pgp_foundUserKeys   : pgpPubTypes.PGPKeySetRef;             (* Result keys set for PGP_USERID search operation  *)
  pgp_foundKeysList   : pgpPubTypes.PGPKeyListRef;            (* List of keys from <pgp_foundUserKeys> keyset     *)
  pgp_keyListIterator : pgpPubTypes.PGPKeyIterRef;            (* Iterator for  <pgp_foundKeysList>                *)
  pgp_signingKey      : pgpPubTypes.PGPKeyRef;                (* Key needed for signing messages                  *)
  pgp_filter          : pgpPubTypes.PGPFilterRef;             (* Filter with PGP_USERID search criterion          *)
  pgp_sigData         : pgpEncode.PGPEventSignatureData;      (* Message signature data                           *)
  pgp_srcFileRef      : pgpPubTypes.PGPFileSpecRef;
  pgp_pgpFileRef      : pgpPubTypes.PGPFileSpecRef;
  pgp_orgFileRef      : pgpPubTypes.PGPFileSpecRef;

PROCEDURE ["C"] PGPEventsHandler ( pgp_context : pgpPubTypes.PGPContextRef; VAR pgp_event : pgpEncode.PGPEvent; userValue : pgpBase.PGPUserValue) : pgpBase.PGPError;
  VAR userData : POINTER TO pgpEncode.PGPEventSignatureData;
BEGIN (* PGPEventsHandler *)
  CASE pgp_event.type OF
    | pgpEncode.kPGPEvent_SignatureEvent :
        userData := userValue; userData^ := pgp_event.data.signatureData;
        userData^.signingKey := pgpPubTypes.kInvalidPGPKeyRef;
  ELSE
  END (* case *);
  RETURN pgpPFLErrors.kPGPError_NoErr;
END PGPEventsHandler;

BEGIN (* PGPTest *)

  IF pgpUtilities.PGPsdkInit() = pgpPFLErrors.kPGPError_NoErr THEN
    pgp_resp := pgpUtilities.PGPNewContext(pgpUtilities.kPGPsdkAPIVersion, pgp_context);
    pgp_resp := pgpKeys.PGPOpenDefaultKeyRings(pgp_context, pgpKeys.kPGPKeyRingOpenFlags_None, pgp_defaultKeyset);
    pgp_resp := pgpKeys.PGPNewUserIDStringFilter(pgp_context, PGP_USERID, pgpKeys.kPGPMatchSubString, pgp_filter);
    pgp_resp := pgpKeys.PGPFilterKeySet(pgp_defaultKeyset, pgp_filter, pgp_foundUserKeys);
    pgp_resp := pgpKeys.PGPCountKeys(pgp_foundUserKeys, pgp_numKeys);
    IF pgp_numKeys > 0 THEN
      pgp_resp := pgpKeys.PGPOrderKeySet(pgp_foundUserKeys, pgpKeys.kPGPAnyOrdering, pgp_foundKeysList);
      pgp_resp := pgpKeys.PGPNewKeyIter(pgp_foundKeysList, pgp_keyListIterator);
      pgp_resp := pgpKeys.PGPKeyIterNext(pgp_keyListIterator, pgp_signingKey);
      pgp_resp := pgpUtilities.PGPNewFileSpecFromFullPath(pgp_context, PGP_SRCFILENAME, pgp_srcFileRef);
      pgp_resp := pgpUtilities.PGPNewFileSpecFromFullPath(pgp_context, PGP_PGPFILENAME, pgp_pgpFileRef);
      pgp_resp := pgpUtilities.PGPNewFileSpecFromFullPath(pgp_context, PGP_ORGFILENAME, pgp_orgFileRef);
      pgp_resp := pgpEncode.PGPEncode(pgp_context,
                                      pgpOptionList.PGPOSignWithKey(pgp_context, pgp_signingKey, pgpOptionList.PGPOLastOption(pgp_context)),
                                      pgpOptionList.PGPOEncryptToKeySet(pgp_context, pgp_foundUserKeys ),
                                      pgpOptionList.PGPOInputFile(pgp_context, pgp_srcFileRef),
                                      pgpOptionList.PGPOOutputFile(pgp_context, pgp_pgpFileRef),
                                      pgpOptionList.PGPOLastOption(pgp_context)
                                      );
      pgp_resp := pgpEncode.PGPDecode(pgp_context,
                                      pgpOptionList.PGPOInputFile(pgp_context, pgp_pgpFileRef),
                                      pgpOptionList.PGPOOutputFile(pgp_context, pgp_orgFileRef),
                                      pgpOptionList.PGPOKeySetRef(pgp_context, pgp_defaultKeyset),
                                      pgpOptionList.PGPOEventHandler(pgp_context, PGPEventsHandler, SYSTEM.ADR(pgp_sigData)),
                                      pgpOptionList.PGPOPassphrase(pgp_context, ""),
                                      pgpOptionList.PGPOLastOption(pgp_context)
                                      );
      IF (pgp_resp = pgpPFLErrors.kPGPError_NoErr) AND (pgp_sigData.checked) THEN
        InOut.WriteString("File is signed. ");
        IF pgp_sigData.verified THEN
          InOut.WriteString("Signature verified.")
        ELSE
          InOut.WriteString("Signature not verified.")
        END (* if *)
      END (* if *);
      pgp_resp := pgpUtilities.PGPFreeFileSpec(pgp_orgFileRef);
      pgp_resp := pgpUtilities.PGPFreeFileSpec(pgp_pgpFileRef);
      pgp_resp := pgpUtilities.PGPFreeFileSpec(pgp_srcFileRef);
      pgp_resp := pgpKeys.PGPFreeKeyIter(pgp_keyListIterator);
      pgp_resp := pgpKeys.PGPFreeKeyList(pgp_foundKeysList);
    END (* if *);
    pgp_resp := pgpKeys.PGPFreeFilter(pgp_filter);
    pgp_resp := pgpKeys.PGPFreeKeySet(pgp_foundUserKeys);
    pgp_resp := pgpKeys.PGPFreeKeySet(pgp_defaultKeyset);
    pgp_resp := pgpUtilities.PGPFreeContext(pgp_context);
    pgp_resp := pgpUtilities.PGPsdkCleanup();
  END (* if *);

END PGPTest.
